import QtQuick
import QtQuick.Layouts
import Quickshell
import Quickshell.Widgets
import "widgets" as Widgets

PanelWindow {
    id: panel
    property var weatherConfig: ({})
    property string sysInfoInterval: "2s"
    property int workspaces: 10

    anchors {
        top: true
        left: true
        right: true
    }

    height: 30
    color: "transparent"

    Rectangle {
        anchors.fill: parent
        color: "#141418"
        ShaderEffect {
            id: shaderEffect
            anchors.fill: parent
            property color baseColor: "#191823"
            property color transparent: Qt.rgba(0, 0, 0, 0)
            property real rectWidth: panel.width
            property real rectHeight: panel.height
            property real cornerRadius: 0
            property real time: 0.0
            fragmentShader: "gradient.frag.qsb"
        }

        RowLayout {
            spacing: 10
            width: parent.width
            height: parent.height
            Layout.fillWidth: true
            Layout.fillHeight: true
            implicitWidth: parent.width
            implicitHeight: parent.height

            Item {
                Layout.fillWidth: true
                Layout.preferredWidth: panel.width / 3
                Layout.maximumWidth: panel.width / 3
                Layout.fillHeight: true

                RowLayout {
                    spacing: 10

                    Widgets.Workspaces {
                        id: workspacesWidget
                        workspaces: workspaces
                    }

                    Widgets.NowPlaying {
                        maxWidth: panel.width / 3 - workspacesWidget.implicitWidth - 26
                    }
                }
            }

            Item {
                Layout.fillWidth: true
                Layout.preferredWidth: panel.width / 3
                Layout.maximumWidth: panel.width / 3
                Layout.fillHeight: true

                RowLayout {
                    anchors.centerIn: parent
                    width: parent.width

                    Widgets.ActiveWindow {
                        Layout.fillWidth: true
                        maxWidth: panel.width / 3
                        Layout.maximumWidth: parent.width
                    }
                }
            }

            Item {
                Layout.fillWidth: true
                Layout.preferredWidth: panel.width / 3
                Layout.maximumWidth: panel.width / 3
                Layout.fillHeight: true

                RowLayout {
                    anchors {
                        right: parent.right
                        top: parent.top
                        bottom: parent.bottom
                    }
                    spacing: 0

                    Widgets.Stats {
                        updateInterval: sysInfoInterval
                    }
                    Widgets.Weather {
                        isAmerican: weatherConfig.isamerican ?? false
                        city: weatherConfig.city ?? "London"
                        updateInterval: weatherConfig.weatherupdateinterval ?? 300000
                    }
                    Widgets.Power {}
                    Widgets.SystemTray {}
                    Widgets.Clock {}
                }
            }
        }
    }
}
