import QtQuick
import QtQuick.Layouts
import Quickshell
import "root:/"

BarWidget {
    id: clockWidget
    Layout.minimumWidth: clockLayout.implicitWidth + 10
    Layout.minimumHeight: clockLayout.implicitHeight
    widgetAnchors.margins: 10

    property bool calendarOpen: false

    Rectangle {
        id: background
        anchors.margins: 0
        anchors.centerIn: parent
        width: clockLayout.implicitWidth + 10
        height: 30
        color: "#40ffffff"
        radius: 0
        opacity: mouseArea.containsMouse ? 0.2 : 0
        Behavior on opacity {
            NumberAnimation {
                duration: 200
            }
        }
    }

    Column {
        id: clockLayout
        anchors.horizontalCenter: parent.right
        anchors.verticalCenter: parent.verticalCenter
        rightPadding: 6
        spacing: 0

        Text {
            id: timeText
            anchors.right: parent.horizontalCenter
            text: {
                let now = Globals.date;
                return now.toLocaleString(Qt.locale(), "HH:mm");
            }
            font.pixelSize: Globals.commonStyles.textFontSizeMedium
            font.family: Globals.commonStyles.fontFamily
            font.weight: Globals.commonStyles.fontWeight
            color: Globals.commonStyles.textColor
            opacity: 1
        }

        Text {
            id: dateText
            anchors.right: parent.horizontalCenter
            text: {
                let now = Globals.date;
                let dayName = now.toLocaleDateString(Qt.locale(), "ddd");
                dayName = dayName.charAt(0).toUpperCase() + dayName.slice(1);
                let day = now.getDate();
                let suffix;
                if (day > 3 && day < 21)
                    suffix = 'th';
                else
                    switch (day % 10) {
                    case 1:
                        suffix = "st";
                        break;
                    case 2:
                        suffix = "nd";
                        break;
                    case 3:
                        suffix = "rd";
                        break;
                    default:
                        suffix = "th";
                    }
                let month = now.toLocaleDateString(Qt.locale(), "MMM");
                return `${dayName}, ${day}${suffix} ${month}`;
            }
            font.pixelSize: Globals.commonStyles.smallTextFontSize
            font.family: Globals.commonStyles.fontFamily
            font.weight: Globals.commonStyles.fontWeight
            color: Globals.commonStyles.textColor
            opacity: 1
        }
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        anchors.margins: 0
        hoverEnabled: true
        onClicked: calendarOpen = !calendarOpen
    }

    Calander {
        calendarOpen: clockWidget.calendarOpen
        clockLayout: clockLayout
    }
}
