import QtQuick
import Quickshell
import Quickshell.Services.Mpris
import Quickshell.Io
import Quickshell.Widgets
import QtQuick.Window
import "root:/"

BarWidget {
    id: root
    property int maxWidth: 100
    implicitHeight: 30
    implicitWidth: mainRect.implicitWidth
    width: mainRect.implicitWidth
    height: 30


    property MprisPlayer player: Mpris.players.values[0] ?? null
    onPlayerChanged: updateProcessState()

    Connections {
        target: root.player
        function onPlaybackStateChanged() {
            updateProcessState();
        }
    }

    function updateProcessState() {
        cavaProcess.running = player?.playbackState === MprisPlaybackState.Playing;
        if (!cavaProcess.running) {
            for (let i = 0; i < visualizerRepeater.count; i++) {
                visualizerRepeater.itemAt(i).height = 0;
            }
        }
    }

    Process {
        id: cavaProcess
        running: false
        command: [Quickshell.shellRoot + "/programs/cvjson", Quickshell.shellRoot + "/programs/cava"]
        stdout: SplitParser {
            onRead: function (line) {
                try {
                    const data = JSON.parse(line);
                    for (let i = 0; i < data.length && i < visualizerRepeater.count; i++) {
                        const newHeight = visualizerRepeater.itemAt(i).height = ((data[i] / 1500) * 30)+ 20;
                    }
                } catch (e) {
                    console.log("Failed to parse cava output:", e);
                }
            }
        }
    }

    property double measuredWidth: {
        if (!root.player)
            return 0;
        let titleWidth = titleText.width
        let artistWidth = artistText.width
        return ((Math.max(titleWidth, artistWidth) / Screen.devicePixelRatio) + artContainer.width + 3) * Screen.devicePixelRatio;
    }

    Repeater {
        id: visualizerRepeater
        model: 15
        z: -1

        Rectangle {
            width: (root.measuredWidth / 15) + 8
            x: artContainer.x + (index * (root.measuredWidth / 15) - 4)
            y: parent.height - height + 25
            height: 10
            color: "#464459"
            opacity: 1
            radius: 10
        }
    }

    Item {
        id: mainRect
        anchors {
            fill: parent
            leftMargin: 8
            rightMargin: 8
        }
        implicitWidth: artContainer.width + titleContainer.width + 48
        visible: root.player?.playbackState === MprisPlaybackState.Playing || root.player?.playbackState === MprisPlaybackState.Paused

        Item {
            id: artContainer
            width: 30
            height: 30
            anchors.left: parent.left
            anchors.leftMargin: 32
            anchors.verticalCenter: parent.verticalCenter

            ClippingWrapperRectangle {
                anchors.fill: parent
                anchors.topMargin: 2
                anchors.rightMargin: 2
                color: "transparent"
                radius: 8
                resizeChild: true
                border.color: "#9900246b"
                border.width: 2

                child: Image {
                    id: artImage
                    source: root.player?.trackArtUrl ?? ""
                    fillMode: Image.PreserveAspectCrop
                    antialiasing: true
                }
            }

            Rectangle {
                anchors.fill: parent
                color: "black"
                opacity: 0.3
                visible: mouseArea.containsMouse
                radius: 8
            }

            Text {
                anchors.centerIn: parent
                anchors.verticalCenterOffset: 2
                text: root.player?.playbackState === MprisPlaybackState.Playing ? "pause" : "play_arrow"
                font.family: Globals.commonStyles.iconFontFamily
                font.pixelSize: Globals.commonStyles.iconFontSizeSmall
                color: Globals.commonStyles.textColor
                opacity: mouseArea.containsMouse ? 1.0 : 0.0
                Behavior on opacity {
                    NumberAnimation {
                        duration: 100
                    }
                }
            }

            MouseArea {
                id: mouseArea
                anchors.fill: parent
                hoverEnabled: true
                cursorShape: Qt.PointingHandCursor
                onClicked: root.player?.togglePlaying()
            }
        }

        Item {
            id: titleContainer
            anchors.left: artContainer.right
            anchors.right: parent.right
            anchors.leftMargin: 16
            anchors.rightMargin: 12
            height: 30

            Text {
                id: titleText
                anchors.left: parent.left
                anchors.bottom: parent.verticalCenter
                anchors.bottomMargin: 4
                text: root.player?.trackTitle || ""
                elide: Text.ElideRight
                color: Globals.commonStyles.textColor
                font.family: Globals.commonStyles.fontFamily
                font.pixelSize: Globals.commonStyles.textFontSizeMedium
                font.weight: Globals.commonStyles.fontWeight
                width: Math.min(implicitWidth, root.maxWidth - artContainer.width - 32)
            }

            Text {
                id: artistText
                anchors.left: parent.left
                anchors.top: parent.verticalCenter
                anchors.topMargin: -2
                text: root.player?.trackArtists || ""
                elide: Text.ElideRight
                color: Globals.commonStyles.textColor
                font.family: Globals.commonStyles.fontFamily
                font.pixelSize: Globals.commonStyles.smallTextFontSize
                font.weight: Globals.commonStyles.fontWeight
                width: Math.min(implicitWidth, root.maxWidth - artContainer.width - 32)
            }
        }
    }
}
